/*************************************************************************\
  "icoutput.h" - Intermediate Code Output (Phase E)
  Revision 6; 10 Nov 2002 - 21 Jan 2003
  Copyright (C) 2002-2003  Kevin P. Albrecht
\*************************************************************************/

#ifndef ICOUTPUT_H
#define ICOUTPUT_H

/////////////////////////
// DEFINES //////////////
/////////////////////////

/////////////////////////
// INCLUDES /////////////
/////////////////////////

#include "icm.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

// External //
void icoutput (char *filename);

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

//// icmoutput
//// Output intermediate code to file
void icoutput (char *filename)
{
  // Vars //
  FILE *fileout;
  Statement *sm;
  int nInstruction = 0;
  char *temp = NULL;

  // Open File //
  fileout = fopen (filename, "w");

  sm = icm_getnext();
  nInstruction++;

  while (sm != NULL)
  {
    if (sm->tac == tacLINE)
    {
      fprintf (fileout, "%d\t%s \"%s\" %d\n", nInstruction, gvsTACStrings[sm->tac], (char *)(sm->a->value), sm->number);
      fprintf (fileout, "\n%s\n", (char *)(sm->b->value));
    }
    else
    {
      fprintf (fileout, "%d\t%s ", nInstruction, gvsTACStrings[sm->tac]);

      if (sm->a)
      {
        if (sm->a->rep == repIdent) fprintf (fileout, "%s ", sm->a->key);
        else
        {
          if (sm->a->type == tLabel)
          {
            temp = conv_is (*((int*)sm->a->value));
            fprintf (fileout, "%s ", temp);
          }
          else if (sm->a->type == tRoutine)
          {
            fprintf (fileout, "%s(%d)", (char *)(sm->a->key), ((Routine*)(sm->a->value))->location );
          }
          else if (sm->a->type == tInt)
          {
            temp = conv_is (*((int*)sm->a->value));
            fprintf (fileout, "%s ", temp);
          }
          else if (sm->a->type == tRat)
          {
            temp = conv_fs (*((float*)sm->a->value));
            fprintf (fileout, "%s ", temp);
          }
          else if (sm->a->type == tStr)
          {
            if (sm->a->value == NULL)
              fprintf (fileout, "\"\" ");
            else
              fprintf (fileout, "\"%s\" ", (char *)(sm->a->value));
          }

          if (temp)
          {
            k_del (temp);
            temp = NULL;
          }
        }
      }
      else fprintf (fileout, "");

      if (sm->b)
      {
        if (sm->b->rep == repIdent) fprintf (fileout, "%s ", sm->b->key);
        else
        {
          if (sm->b->type == tLabel)
          {
            temp = conv_is (*((int*)sm->b->value));
            fprintf (fileout, "%s ", temp);
          }
          else if (sm->b->type == tRoutine)
          {
            fprintf (fileout, "%s(%d)", (char *)(sm->b->key), ((Routine*)(sm->b->value))->location );
          }
          else if (sm->b->type == tInt)
          {
            temp = conv_is (*((int*)sm->b->value));
            fprintf (fileout, "%s ", temp);
          }
          else if (sm->b->type == tRat)
          {
            temp = conv_fs (*((float*)sm->b->value));
            fprintf (fileout, "%s ", temp);
          }
          else if (sm->b->type == tStr)
          {
            if (sm->b->value == NULL)
              fprintf (fileout, "\"\" ");
            else
              fprintf (fileout, "\"%s\" ", (char *)(sm->b->value));
          }

          if (temp)
          {
            k_del (temp);
            temp = NULL;
          }
        }
      }
      else fprintf (fileout, "");

      if (sm->c)
      {
        if (sm->c->rep == repIdent) fprintf (fileout, "%s ", sm->c->key);
        else
        {
          if (sm->c->type == tLabel)
          {
            temp = conv_is (*((int*)sm->c->value));
            fprintf (fileout, "%s ", temp);
          }
          else if (sm->c->type == tRoutine)
          {
            fprintf (fileout, "%s(%d)", (char *)(sm->c->key), ((Routine*)(sm->c->value))->location );
          }
          else if (sm->c->type == tInt)
          {
            temp = conv_is (*((int*)sm->c->value));
            fprintf (fileout, "%s ", temp);
          }
          else if (sm->c->type == tRat)
          {
            temp = conv_fs (*((float*)sm->c->value));
            fprintf (fileout, "%s ", temp);
          }
          else if (sm->c->type == tStr)
          {
            if (sm->c->value == NULL)
              fprintf (fileout, "\"\" ");
            else
              fprintf (fileout, "\"%s\" ", (char *)(sm->c->value));
          }

          if (temp)
          {
            k_del (temp);
            temp = NULL;
          }
        }
      }
      else fprintf (fileout, "");

      if (sm->number != 0) fprintf (fileout, "%d\n", sm->number);
      else                 fprintf (fileout, "\n");
    }
    
    sm = icm_getnext();
    nInstruction++;
  }

  // Close File //
  fclose (fileout);
}

#endif
